﻿using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class RoutingModelWithKinematicWaveFactory : McBasicModelFactoryBase
	{
		/// <summary><para>method outline:</para>
		/// <para>デフォルトコンストラクタ</para>
		/// </summary>
		/// <param name="">無し</param>
		/// <returns>インスタンス</returns>
		/// <exception cref="">無し</exception>
        public RoutingModelWithKinematicWaveFactory()
		{
		}

		/// <summary><para>method outline:</para>
		/// <para>ファクトリ識別子を生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HySID csFactID = CreateFactoryID( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>ファクトリ識別子</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override HySID CreateFactoryID()
		{
            return RoutingModelWithKinematicWaveDefine.HYDROLOGY_MODEL_LIB;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算モデルクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalModel csCalModel = CreateCalModel(csModelKind) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>演算モデルクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalModel CreateCalModel(HySObjectKind csModelKind)
		{
			McCalModel csCalModel = null;
            if (csModelKind == RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_KIND)
			{
                csCalModel = new RoutingModelWithKinematicWaveModel();
			}
			else
			{
			}
			
			return csCalModel;
		}

		/// <summary><para>method outline:</para>
		/// <para>要素内演算中データクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalInfo csCalModel = CreateCalInfo( csModelKind ) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>要素内演算中データクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalInfo CreateCalInfo(HySObjectKind csModelKind)
		{
			McCalInfo csCalInfoData = null;
            if (csModelKind == RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_KIND)
			{
                csCalInfoData = new RoutingModelWithKinematicWaveInfo();
			}
			else
			{
			}
			
			return csCalInfoData;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルプロパティ表示／設定情報クラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csPropertyInf = CreateModelPropertyInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>モデルプロパティ表示／設定情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCellModelPropertyIF CreateModelProperty(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McCellModelPropertyInfo csRtnCellPrptyDt = null;

            if (csModelKind == RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
				csRtnCellPrptyDt.SetStepTime((double)60); // δT設定 60秒

				csRtnCellPrptyDt.AddInfoType("mL", "河道長 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("mL", 800.0);

				csRtnCellPrptyDt.AddInfoType("mB", "河道幅 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("mB", 1.0);
				csRtnCellPrptyDt.AddInfoType("mG", "河道勾配 (tangent の値)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("mG", 0.00418);
				csRtnCellPrptyDt.AddInfoType("mMan", "マニングの粗度係数 (m-s)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("mMan", 0.03);

				// ここ重要　後の変数宣言に使う
				csRtnCellPrptyDt.AddInfoType("mDiv", "区間分割数 (-)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("mDiv", 20);

				// csRtnCellPrptyDt.AddInfoType("mDt", "時間差分間隔 (sec)", McDefine.ValKind.LONG);
				// csRtnCellPrptyDt.SetInfo("mDt", 20);

				csRtnCellPrptyDt.AddInfoType("mTheta", "差分重み係数 (-)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("mTheta", 0.65);


				// mDivの数だけパラメータ記入の仮仕様
				for (int lDiv = 0; lDiv <= 20; lDiv++)
				{
					csRtnCellPrptyDt.AddInfoType("mK-" + lDiv.ToString(), "R = mK * pow(A, mZ) (m系)によるmk" + lDiv.ToString(), McDefine.ValKind.DOUBLE);
					csRtnCellPrptyDt.SetInfo("mK-" + lDiv.ToString(), 0.0171937);
					csRtnCellPrptyDt.AddInfoType("mZ-" + lDiv.ToString(), "R = mK * pow(A, mZ) (m系)によるmZ" + lDiv.ToString(), McDefine.ValKind.DOUBLE);
					csRtnCellPrptyDt.SetInfo("mZ-" + lDiv.ToString(), 1.0);
				}
				

				{
					// 第1パターン
					McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                        RoutingModelWithKinematicWaveDefine.IN_PATTERN_TOP,
						McTranInfoDefine.SINGLE_CELL_SERIAL,
						"上流端流入") as McTranInfoPattern;
					{
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec"); 
						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
						csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR);
					}
					csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

					// 第2パターン
					csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                        RoutingModelWithKinematicWaveDefine.IN_PATTERN_LATERAL,
						McTranInfoDefine.SINGLE_CELL_SERIAL,
						"支流からの横流入") as McTranInfoPattern;
					{
						// セル内変数設定
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec"); 
						}
						
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
						csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR);
					}
					csRtnCellPrptyDt.AddReceivePattern(csTrnPtn); 
				}

				{
					McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                        RoutingModelWithKinematicWaveDefine.OUT_PATTERN_BOTTOM,
						McTranInfoDefine.SINGLE_CELL_SERIAL,
						"下流端出力") as McTranInfoPattern;
					{
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec");
							// csCellChara.SetDataKind(1, "水位", HySDataKindDefine.WATER_LEVEL, "m");
							// csCellChara.SetDataKind(2, "流速", HySDataKindDefine.SCALOR_VELOCITY, "m/sec");
						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
					}
					csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
				}
			}
			else
			{
				// Do nothing
			}

			return csRtnCellPrptyDt;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル初期化表示／設定情報生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csInitialInf = CreateModelInitialInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>初期情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McPropertyInfoRoot CreateModelInitialInfo(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McInitialInfo csRtnDt = null;

            if (csModelKind == RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);

				for (int lDiv = 0; lDiv <= 20; lDiv++)
				{
					csRtnDt.AddInfoType("mQc-" + lDiv.ToString(), "セル" +lDiv.ToString() + "番目の流量", McDefine.ValKind.DOUBLE);
					csRtnDt.SetInfo("mQc-" + lDiv.ToString(), 0.0);
				}
			}
			else
			{
				// Do Nothing
			}

			return csRtnDt;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル情報提供</para>
		/// </summary>
		/// <example>
		/// <para>usage:</para>
		/// <para>HySDataLinkedList csModelInfoList = GetCalModelInfoList();</para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>モデル情報のリスト</returns>
		/// <exception cref="">無し</exception>
		/// <remarks>
		/// <para>remarks:</para>
		/// <para>モデル情報(McModelInfo)をリスト形式で返却する</para>
		/// </remarks>
		public override HySDataLinkedList GetCalModelInfoList()
		{
			McModelInfo csModelInfo=null;

			csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
                    // McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY, 
                    new HySObjectKind("CAL_RIVER_MODELS"),
                    RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_KIND,
                    RoutingModelWithKinematicWaveDefine.HYDROLOGY_RIVER_KINEMATIC_WAVE_NAME);
			csModelInfo.SetVersionInf("Ver1.0");
			csModelInfo.SetSummaryInf("KyotoUnivHydrologyのKinematicWave法による河道流出モデルです");
			csModelInfo.SetCreatorInf("京都大学大学院 水文水資源学研究室");
			csModelInfo.SetIconName("Lane");
			m_csCalModelInforList.AddLast(csModelInfo);

			return m_csCalModelInforList;
		}

	}
}


namespace CommonMP.HYMCO._SYSTEM_
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class HYMCO_MODEL_FACTORY_CREATOR_IMPLE : HYMCO_MODEL_FACTORY_CREATOR
	{
		/// <summary><para>method outline:</para>
		/// <para>コンストラクター</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HYMCO_MODEL_FACTORY_CREATOR_IMPLE csFactCrt = new HYMCO_MODEL_FACTORY_CREATOR_IMPLE( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>生成インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public HYMCO_MODEL_FACTORY_CREATOR_IMPLE()
		{
		}
		/// <summary><para>method outline:</para>
		/// <para>演算モデルファクトリクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McModelFactory csCalModelFactory = CreateFactory( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>演算モデルファクトリクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public CommonMP.HYMCO.Interface.Model.McModelFactory CreateFactory()
		{
            return new KyotoUnivEngHywr.RoutingModelWithKinematicWaveFactory();
		}
	}
}