﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

using CommonMP.HYMCO.CoreImpl.Data;


namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUnivRiverKinematicWaveModel 河道流追跡データクラス</para>
	/// </summary>
	[Serializable]
    public class RoutingModelWithKinematicWaveInfo : McCalInfo
	{
		/// <summary> R = mK * pow(A, mZ) (m系)</summary>
		public double[] mK;
		/// <summary>R = mK * pow(A, mZ) (m系)</summary>
		public double[] mZ;
		/// <summary>マニングの粗度係数 (m-s) </summary>
		public double mMan;
		/// <summary> 河道長 (m)</summary>
		public double mL;
		/// <summary> 河道幅 (m)</summary>
		public double mB;
		/// <summary>  河道勾配 (tangent の値)</summary>
		public double mG;
		/// <summary>区間分割数 (-) </summary>
		public long mDiv;
		/// <summary> 空間差分間隔 (m)</summary>
		public double mDx;
		/// <summary>時間差分間隔 (sec) </summary>
		public long mDt;
		/// <summary> 差分重み係数 (-)</summary>
		public double mTheta;

		/// <summary> kinematic 定数 
		///  mAlpha = 1 / mMan * pow(mK, 2/3) * sqrt(mG)
		///  </summary>
		public double[] mAlpha;
		/// <summary> kinematic 定数
		/// mM = 1 + 2/3*mZ 
		/// </summary>
		public double[] mM;

		// 状態量

		/// <summary>current_time での流量 (m3/s) </summary>
		public double[] mQc;
		/// <summary>current_time+time_step での流量 (m3/s) </summary>
		public double[] mQn;
		/// <summary>current_time での通水断面積 (m2) </summary>
		public double[] mAc;
		/// <summary> current_time+time_step での通水断面積 (m2)</summary>
		public double[] mAn;
		/// <summary>current_time での側方流入量 (m2/s) </summary>
		public double mQlc;
		/// <summary>current_time+time_step での側方流入量 (m2/s) </summary>
		public double mQln;

		// その他
		/// <summary> 河道流量の初期値を線形内挿する場合：0  読み込む場合：1</summary>
		public int initFlag;

		// 受信端子・送信端子

		/// <summary>側方流入量の数 </summary>
		public long mNumLateral;
		/// <summary>河道上端からの流入数 </summary>
		public long mNumUpInflow;


		/// <summary>最後に伝送した演算時刻 </summary>
		public HySTime m_csLastSendTime = new HySTime(0);

		/// <summary> </summary>
		public void Initialize_mDiv(long iDiv)
		{
			mDiv = iDiv;
			mK = new double[iDiv + 1];
			mZ = new double[iDiv + 1];
			mAlpha = new double[iDiv + 1];
			mM = new double[iDiv + 1];

			mQc = new double[iDiv + 1];
			mQn = new double[iDiv + 1];
			mAc = new double[iDiv + 1];
			mAn = new double[iDiv + 1];
			mQlc = 0.0;
			mQln = 0.0;
		}

		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
            RoutingModelWithKinematicWaveInfo csRtn = new RoutingModelWithKinematicWaveInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
            RoutingModelWithKinematicWaveInfo csOrdDt = (RoutingModelWithKinematicWaveInfo)csOrgInfo;
			
			// this.m_dData = csOrdDt.m_dData;
			this.m_csLastSendTime.SetTime(csOrdDt.m_csLastSendTime);

			this.Initialize_mDiv(csOrdDt.mDiv);

			this.mMan = csOrdDt.mMan;
			this.mL = csOrdDt.mL;
			this.mB = csOrdDt.mB;
			this.mG = csOrdDt.mG;
			this.mDx = csOrdDt.mDx;

			this.mDt = csOrdDt.mDt;
			this.mTheta = csOrdDt.mTheta;

			this.mNumLateral = csOrdDt.mNumLateral;
			this.mNumUpInflow = csOrdDt.mNumUpInflow;

			for (int i = 0; i <= mDiv; i++)
			{
				this.mK[i] = csOrdDt.mK[i];

				this.mZ[i] = csOrdDt.mZ[i];
				this.mAlpha[i] = csOrdDt.mAlpha[i];
				this.mM[i] = csOrdDt.mM[i];
				this.mQc[i] = csOrdDt.mQc[i];
				this.mQn[i] = csOrdDt.mQn[i];
				this.mAc[i] = csOrdDt.mAc[i];
				this.mAn[i] = csOrdDt.mAn[i];
			}
			this.mQlc = csOrdDt.mQlc;
			this.mQln = csOrdDt.mQln;
			
			return true;
		}

	}
}
